from naas_drivers.driver import InDriver, OutDriver
from .__crud import CRUD
import requests
import os


class Users(CRUD):
    def get_by_email(self, email):
        data = {
            "apiKey": self.auth,
            "email": email,
        }
        req = requests.post(
            url=f"{self.base_public_url}/{self.endpoint}/retrieve",
            headers=self.req_headers,
            json=data,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req

    def get(self, uid):
        data = {
            "apiKey": self.auth,
            "id": uid,
        }
        req = requests.post(
            url=f"{self.base_public_url}/{self.endpoint}/retrieve",
            headers=self.req_headers,
            json=data,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req

    def send(self, email, name=None):
        data = {
            "apiKey": self.auth,
            "email": email,
        }
        if name:
            data["name"] = name
        req = requests.post(
            url=f"{self.base_public_url}/{self.endpoint}/find_or_create",
            headers=self.req_headers,
            json=data,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req

    def delete(self, uid):
        data = {
            "apiKey": self.auth,
            "id": uid,
        }
        req = requests.post(
            url=f"{self.base_public_url}/{self.endpoint}/delete",
            headers=self.req_headers,
            json=data,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req


class Canny(InDriver, OutDriver):
    base_url = os.environ.get("CANNY_URL", "https://canny.io")
    api_url = None

    def __init__(self):
        self.api_url = f"{self.base_url}/api/v1"

    def connect(self, api_token):
        self.key = api_token
        self.connected = True
        self.users = Users(self.api_url, "users", self.key)
        return self



import os
import subprocess


class Streamlit:
    """
    Naas Streamlit app
    """

    def add(self, path="app.py", port=9999, debug=True):
        self.path = path
        self.port = port
        os.system(f"fuser -n tcp -k {self.port}")
        cmd = (
            f"streamlit run {self.path} --server.port {self.port}  --logger.level debug"
        )
        with subprocess.Popen(
            [cmd],
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            bufsize=1,
            universal_newlines=True,
        ) as proc:
            print(cmd)
            print(proc)
            username = os.environ.get("JUPYTERHUB_USER", None)
            if username:
                print(
                    f"\n🎉 Streamlit application is accessible here: 'https://app.naas.ai/user/{username}/proxy/{port}/'"
                )

            while proc.returncode is None:
                line = proc.stdout.readline()
                if debug:
                    print(line.rstrip())
                line = proc.stderr.readline()
                if debug:
                    print(line.rstrip())



"""Google Analytics Driver."""
import re
from datetime import datetime, timedelta
import numpy as np
import pandas as pd
from google.oauth2 import service_account
from apiclient.discovery import build
from naas_drivers.driver import InDriver, OutDriver


# Helper function
def ga_naming_to_title(ga_nanimg: str):
    name = ga_nanimg.split(":")[-1]
    splited_name = re.findall(r"[A-Z]?[a-z]+|[A-Z]+(?=[A-Z]|$)", name)
    return " ".join([name.title() for name in splited_name])


ga_metrics = [
    "ga:users",
    "ga:newUsers",
    "ga:percentNewSessions",
    "ga:sessions",
    "ga:bounces",
    "ga:bounceRate",
    "ga:sessionDuration",
    "ga:avgSessionDuration",
    "ga:organicSearches",
    "ga:entrances",
    "ga:entranceRate",
    "ga:pageviews",
    "ga:pageviewsPerSession",
    "ga:uniquePageviews",
    "ga:timeOnPage",
    "ga:avgTimeOnPage",
    "ga:exits",
    "ga:exitRate",
    "ga:pageLoadTime",
    "ga:pageLoadSample",
    "ga:avgPageLoadTime",
    "ga:domainLookupTime",
    "ga:avgDomainLookupTime",
    "ga:pageDownloadTime",
    "ga:avgPageDownloadTime",
    "ga:redirectionTime",
    "ga:avgRedirectionTime",
    "ga:serverConnectionTime",
    "ga:avgServerConnectionTime",
    "ga:serverResponseTime",
    "ga:avgServerResponseTime",
    "ga:speedMetricsSample",
    "ga:domInteractiveTime",
    "ga:avgDomInteractiveTime",
    "ga:domContentLoadedTime",
    "ga:avgDomContentLoadedTime",
    "ga:domLatencyMetricsSample",
    "ga:uniqueDimensionCombinations",
    "ga:hits",
    "ga:sessionsPerUser",
]


class GoogleAnalytics(InDriver, OutDriver):
    """
    Google Analytics driver.
    """

    def __init__(self) -> None:
        self.views = Views(self)
        self.available_metrics = []
        self.__generate_methods()

    def connect(self, json_path: str, view_id: str):
        credentials = service_account.Credentials.from_service_account_file(
            json_path, scopes=["https://www.googleapis.com/auth/analytics.readonly"]
        )
        self.view_id = view_id
        self.service = build("analyticsreporting", "v4", credentials=credentials)
        return self

    # This method is used to automatically generate methods based on metrics available in GA.
    def __generate_methods(self):
        for metric_id in ga_metrics:
            computed_name = metric_id
            computed_name = re.sub(r"(?<!^)(?=[A-Z])", "_", computed_name).lower()[3:]

            try:
                getattr(self, computed_name)
            except Exception as e:  # noqa: F841
                self.available_metrics.append(computed_name)
                setattr(self, computed_name, Metric())

            def custom_get_trend(metric_id):
                return (
                    lambda dimensions, start_date=None, end_date=None: self.get_trend(
                        metric_id, dimensions, start_date, end_date
                    )
                )

            setattr(
                getattr(self, computed_name), "get_trend", custom_get_trend(metric_id)
            )

    def get_trend(self, metrics, dimensions, start_date, end_date):
        """
        Return an dataframe object with 6 columns:
        - DATE         GA dimensions
        - METRIC       GA metrics
        - VALUE        Metrics value
        - VALUE_COMP   Metrics last value comparison
        - VARV         Variation in value between VALUE and VALUE_COMP
        - VARP         Variation in % between VALUE and VALUE_COMP

        Parameters
        ----------
        metrics: str:
            - New visitors = "ga:newUsers"
            - User = "ga:users"
        dimensions: str:
            List of google analytics dimensions
            - Hourly = "hourly"
            - Day = "daily"
            - Week = "weekly"
            - Month = "monthly"
        start_date: str: default="30daysAgo"
            "NdaysAgo" with n equal to int
        end_date: str: default="today"
            "today", "yesterday"
        """

        allowed_aliases = ["hourly", "daily", "weekly", "monthly"]
        allowed_dimensions = [
            "ga:date,ga:hour",
            "ga:date",
            "ga:year,ga:week",
            "ga:year,ga:month",
        ]

        if dimensions in allowed_aliases:
            dimensions = allowed_dimensions[allowed_aliases.index(dimensions)]

        if dimensions not in allowed_dimensions:
            raise Exception(f"'dimensions' should be one of {allowed_dimensions}")

        # Get data
        df = self.views.get_data(
            self.view_id,
            metrics=metrics,
            dimensions=dimensions,
            start_date=start_date,
            end_date=end_date,
            format_type="summary",
            pivots_dimensions="ga:country",  # not used
        )

        # Format trend dataset
        df["DATE_ISO"] = df.index
        df = df.reset_index(drop=True)
        if dimensions == "ga:date,ga:hour":
            df["DATE_ISO"] = pd.to_datetime(
                df.apply(lambda row: f"{row.DATE_ISO[0]} {row.DATE_ISO[1]}:00", axis=1)
            )
            df["DATE"] = df["DATE_ISO"].dt.strftime("%Y-%m-%d %H:00:00")
        elif dimensions == "ga:date":
            df["DATE_ISO"] = pd.to_datetime(
                df.apply(lambda row: row.DATE_ISO[0], axis=1)
            )
            df["DATE"] = df["DATE_ISO"].dt.strftime("%Y-%m-%d")
        elif dimensions == "ga:year,ga:week":
            df["DATE_ISO"] = pd.to_datetime(
                df.apply(
                    lambda row: datetime.strptime(
                        f"{row.DATE_ISO[0]}-W{row.DATE_ISO[1]}" + "-1", "%Y-W%W-%w"
                    ),
                    axis=1,
                )
            )
            df["DATE"] = df["DATE_ISO"].dt.strftime("%Y W%W")
        elif dimensions == "ga:year,ga:month":
            df["DATE_ISO"] = pd.to_datetime(
                df.apply(
                    lambda row: datetime.strptime(
                        f"{row.DATE_ISO[0]}-M{row.DATE_ISO[1]}", "%Y-m%m"
                    ),
                    axis=1,
                )
            )
            df["DATE"] = df["DATE_ISO"].dt.strftime("%Y %b")
        df["METRIC"] = metrics.replace("ga:", "")
        df["VALUE"] = df[metrics]
        df = df.drop(metrics, axis=1)
        df.columns = df.columns.str.upper()

        # Calc variation
        for idx, row in df.iterrows():
            if idx == 0:
                value_n1 = 0
            else:
                value_n1 = df.loc[df.index[idx - 1], "VALUE"]
            df.loc[df.index[idx], "VALUE_COMP"] = value_n1
        df["VARV"] = df["VALUE"] - df["VALUE_COMP"]
        df["VARP"] = df["VARV"] / abs(df["VALUE_COMP"])
        return df


class Metric:
    pass


class Views:
    def __init__(self, parent) -> None:
        self.parent = parent

    @staticmethod
    def _get_body(
        view_id: str,
        start_date: str,
        end_date: str,
        metrics: str,
        pivots_dimensions: str,
        dimensions: str = "ga:yearMonth",
        max_group_count: int = 1000,
    ) -> dict:
        """
        Create the body of the request to Google Analytics Reporting API V4.

        Args:
            view_id: your access point for reports; a defined view of data from a property.
            date_ranges: e.g. [{"startDates": "2020-01-01", "endDates": "2020-12-31"}]
            metrics: e.g. [{'expression': 'ga:users'}, {"expression": "ga:bounceRate"}]
            pivot_dimension: e.g. [{"name": "ga:channelGrouping"}]
            dimensions: e.g. [{'name': 'ga:yearMonth'}]

        Returns response in JSON format.
        """
        return {
            "reportRequests": [
                {
                    "viewId": view_id,
                    "dateRanges": {"startDate": start_date, "endDate": end_date},
                    "metrics": [
                        {"expression": metric} for metric in metrics.split(",")
                    ],
                    "dimensions": [
                        {"name": dimension} for dimension in dimensions.split(",")
                    ],
                    "pivots": [
                        {
                            "dimensions": {"name": pivots_dimensions},
                            "metrics": [{"expression": metrics}],
                            "maxGroupCount": max_group_count,
                        }
                    ],
                }
            ]
        }

    def get_data(
        self,
        view_id: str,
        metrics: str,
        pivots_dimensions: str,
        dimensions: str = "ga:yearMonth",
        start_date: str = None,
        end_date: str = None,
        format_type: str = "summary",
        max_group_count: int = 1000,
    ) -> pd.DataFrame:
        """
        Get data from Google Analytics Reporting API V4.
        """
        if format_type not in ("summary", "pivot"):
            raise ValueError(
                f"format_type must be either <summary> or <pivot> but is: {format_type}"
            )
        # Default date values
        start_date = (
            start_date
            if start_date
            else (datetime.today() - timedelta(days=365)).strftime("%Y-%m-%d")
        )
        end_date = end_date if end_date else datetime.today().strftime("%Y-%m-%d")
        # Create body
        body = self._get_body(
            view_id,
            start_date,
            end_date,
            metrics,
            pivots_dimensions,
            dimensions,
            max_group_count=max_group_count,
        )
        # Fetch Data
        try:
            response = self.parent.service.reports().batchGet(body=body).execute()
        except Exception as error:
            raise error()
        # JSON to Pandas DataFrame
        if format_type == "summary":
            return self.format_summary(response)
        return self.format_pivot(response)

    def get_unique_visitors(
        self, view_id: str, start_date: str = None, end_date: str = None
    ) -> pd.DataFrame:
        """
        Get the number of unique visitors.
        """
        unique_visitors = self.get_data(
            view_id,
            metrics="ga:users",
            pivots_dimensions="ga:channelGrouping",
            dimensions="ga:yearMonth",
            start_date=start_date,
            end_date=end_date,
            format_type="summary",
        )
        unique_visitors.reset_index(inplace=True)
        unique_visitors.columns = [
            ga_naming_to_title(col) for col in unique_visitors.columns
        ]
        return unique_visitors

    def get_bounce_rate(
        self, view_id=str, start_date: str = None, end_date: str = None
    ) -> pd.DataFrame:
        """
        Get the number of unique visitors.
        """
        bounce_rate = self.get_data(
            view_id,
            metrics="ga:bounceRate",
            pivots_dimensions="ga:channelGrouping",
            dimensions="ga:yearMonth",
            start_date=start_date,
            end_date=end_date,
            format_type="summary",
        )
        bounce_rate.reset_index(inplace=True)
        bounce_rate.columns = [ga_naming_to_title(col) for col in bounce_rate.columns]
        bounce_rate["Bounce Rate"] /= 100
        return bounce_rate

    def get_time_landing(
        self,
        view_id: str,
        landing_path: str = "/",
        start_date: str = None,
        end_date: str = None,
    ) -> pd.DataFrame:
        """
        Get the average time on landing page.
        """
        avg_time_landing = self.get_data(
            view_id,
            metrics="ga:avgTimeOnPage",
            pivots_dimensions="ga:landingPagePath",
            dimensions="ga:yearMonth",
            start_date=start_date,
            end_date=end_date,
            format_type="pivot",
        )
        if landing_path in avg_time_landing.columns:
            avg_time_landing = avg_time_landing.loc[:, landing_path]
        else:
            raise KeyError(
                f"Landing Path ({landing_path}) is not an available url pattern."
            )
        avg_time_landing.index.rename("Year Month", inplace=True)
        avg_time_landing.rename(
            columns={"ga:avgTimeOnPage": "avg_time_landing"}, inplace=True
        )
        avg_time_landing.reset_index(inplace=True)
        return avg_time_landing

    def get_pageview(
        self, view_id: str, start_date: str = None, end_date: str = None
    ) -> pd.DataFrame:
        """
        Get the views of pages.
        """
        pageview = self.get_data(
            view_id,
            metrics="ga:pageviews",
            pivots_dimensions="ga:pagePath",
            dimensions="ga:year",
            start_date=start_date,
            end_date=end_date,
            format_type="pivot",
        )
        pageview.columns = [page[0] for page in pageview.columns]
        pageview = pageview.head(1).T
        pageview.reset_index(inplace=True)
        pageview.columns = ["Pages", "Pageview"]
        return pageview

    def get_country(
        self,
        view_id: str,
        metrics: str = "ga:sessions",
        start_date: str = None,
        end_date: str = None,
    ):
        """
        Get sessions per country.
        """
        country = self.get_data(
            view_id,
            metrics=metrics,
            pivots_dimensions="ga:country",
            dimensions="ga:year",
            start_date=start_date,
            end_date=end_date,
            format_type="pivot",
        )
        country.columns = [ga_naming_to_title(c[0]) for c in country.columns]
        country = country.T
        country.reset_index(inplace=True)
        country.columns = ["Country", ga_naming_to_title(metrics)]
        return country

    @staticmethod
    def format_summary(response):
        """
        Format summary table.
        """
        row_index_names = response["reports"][0]["columnHeader"]["dimensions"]
        row_index = [
            element["dimensions"] for element in response["reports"][0]["data"]["rows"]
        ]
        row_index_named = pd.MultiIndex.from_arrays(
            np.transpose(np.array(row_index)), names=np.array(row_index_names)
        )
        # extract column names
        summary_column_names = [
            item["name"]
            for item in response["reports"][0]["columnHeader"]["metricHeader"][
                "metricHeaderEntries"
            ]
        ]
        # extract table values
        summary_values = [
            element["metrics"][0]["values"]
            for element in response["reports"][0]["data"]["rows"]
        ]
        return pd.DataFrame(
            data=np.array(summary_values),
            index=row_index_named,
            columns=summary_column_names,
        ).astype("float")

    @staticmethod
    def format_pivot(response):
        """
        Creates the final dataframe.
        """
        # extract table values
        pivot_values = [
            item["metrics"][0]["pivotValueRegions"][0]["values"]
            for item in response["reports"][0]["data"]["rows"]
        ]
        # create column index
        top_header = [
            item["dimensionValues"]
            for item in response["reports"][0]["columnHeader"]["metricHeader"][
                "pivotHeaders"
            ][0]["pivotHeaderEntries"]
        ]
        column_metrics = [
            item["metric"]["name"]
            for item in response["reports"][0]["columnHeader"]["metricHeader"][
                "pivotHeaders"
            ][0]["pivotHeaderEntries"]
        ]
        array = np.concatenate(
            (
                np.array(top_header),
                np.array(column_metrics).reshape((len(column_metrics), 1)),
            ),
            axis=1,
        )
        column_index = pd.MultiIndex.from_arrays(np.transpose(array))
        # create row index
        row_index_names = response["reports"][0]["columnHeader"]["dimensions"]
        row_index = [
            element["dimensions"] for element in response["reports"][0]["data"]["rows"]
        ]
        row_index_named = pd.MultiIndex.from_arrays(
            np.transpose(np.array(row_index)), names=np.array(row_index_names)
        )
        # combine into a dataframe
        return pd.DataFrame(
            data=np.array(pivot_values), index=row_index_named, columns=column_index
        ).astype("float")



from naas_drivers.driver import InDriver, OutDriver
import urllib.parse
from git import Repo


class Git(InDriver, OutDriver):
    def connect(self, config):
        """
        Description: This class accepst a JSON configuration and executes respective
        function as per the action specified in the configuration
        """

        self.config = dict(config)

        if [
            "username",
            "password",
            "github_url",
            "branch",
            "target_folder",
            "action",
            "commit_message",
        ] != [i for i in self.config.keys()]:
            print(
                "Configuration missing mandatory parameters (username,password,github_url,branch,target_folder,action)"
            )
            return

        if config.get("action", None) == "clone":
            self.clone(
                dest_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
                repourl=self.config.get("github_url", None),
                username=self.config.get("username", None),
                password=self.config.get("password", None),
            )
        elif config.get("action", None) == "commit":
            self.commit(
                from_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
                message=self.config.get("commit_message", None),
            )
        elif config.get("action", None) == "push":
            self.push(
                from_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
            )
        elif config.get("action", None) == "pull":
            self.pull(
                from_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
            )
        elif config.get("action", None) == "checkout":
            self.checkout(
                from_dir=self.config.get("target_folder", None),
                branch=self.config.get("branch", None),
            )
        self.connected = True
        return self

    def clone(
        self,
        dest_dir="gitSource",
        branch="master",
        repourl=None,
        username=None,
        password=None,
    ):
        """Clone the repo (dest_dir='gitSource', branch="master", repourl=None, username=None, password=None)"""
        self.check_connect()
        if repourl[0:3] == "git":
            Repo.clone_from(repourl, dest_dir, branch=branch)
        else:
            urlsplit = repourl.split("//")
            authstring = username + ":" + urllib.parse.quote(password)
            urlsplit[1] = authstring + "@" + urlsplit[1]
            repourl = "//".join(urlsplit)
            Repo.clone_from(repourl, dest_dir, branch=branch)
        print("Clone successful")

    def commit(self, from_dir, branch, message="Auto commit"):
        """Commit the files commit(from_dir,branch,message="Auto Commit Message")"""
        self.check_connect()
        repo = Repo(from_dir)
        repo.git.execute(["git", "add", "--all"])
        repo.index.commit(message)
        repo.git.clean("-xdf")
        print("Commit successful!")
        return repo.remote()

    def push(self, from_dir, branch="master"):
        """Push to the specified repository push(self,from_dir,branch="master")"""
        self.check_connect()
        self.checkout(from_dir, branch)
        origin = self.commit(from_dir, branch)
        origin.push()
        print("Push successful")

    def pull(self, from_dir, branch="master"):
        """Pull from the specified repository pull(from_dir,branch="master")"""
        self.check_connect()
        self.checkout(from_dir, branch)
        origin = self.commit(from_dir, branch)
        origin.pull()
        print("Pull successful!")

    def checkout(self, from_dir, branch="master"):
        """Checkout to a spcified branch (from_dir, branch="master")"""
        self.check_connect()
        repo = Repo(from_dir)
        if repo.active_branch != branch:
            repo.git.clean("-xdf")
            repo.git.checkout(branch)
        print("Checkout successful")



# from naas_drivers.gsheet import Gsheet

# SPREADSHEET_ID = "10NHJf8Ny1x9dfg30jLpYPmg9o1UBp5ylwWxKt5EtBH0"
# SHEET_ID = "TSLA"


def test_gsheet():
    # gs = Gsheet()
    # df = gs(spreadsheet_id=SPREADSHEET_ID, sheet_id=SHEET_ID)
    # assert not df.empty
    assert True



from naas_drivers.driver import InDriver, OutDriver
import pandas as pd
import requests
import os
import string
from datetime import datetime


class TKCRUD:
    # class TKCRUD(CRUD):
    def __init__(self, base_url, subdomain, auth):
        self.req_headers = {
            "X-Auth-API-Key": auth,
            "X-Auth-Subdomain": subdomain,
            "Accept": "application/json",
            "Content-Type": "application/json",
        }
        self.base_url = base_url
        self.model_name = self.base_url.split("/")[-1]

        # Manage message
        message_dict = {
            "users": "User",
            "enrollments": "Enrollment",
            "courses": "Course",
            "groups": "Group",
            "group_users": "User group",
        }
        self.msg = message_dict[self.model_name]

    def __values_format(self, data):
        for key, value in data.items():
            # Check if value is not None
            if value is not None:
                # Force format to string  & delete space
                value = str(value).strip()

                # Specific rules
                if key == "first_name":
                    value = value.replace("-", " ")
                    value = string.capwords(value).replace(" ", "-")
                elif key == "last_name":
                    value = value.upper()
                elif key in ["activated_at", "expiry_date"]:
                    try:
                        value = datetime.strptime(value, "%d/%m/%Y")
                        value = value.strftime("%Y-%m-%dT%H:%M:%S.000Z")
                    except ValueError:
                        print(
                            f"'{value}' is not in the correct format.\n"
                            f"Please change it to %d/%m/%Y.\n"
                        )
                # Change value in dict
                data[key] = value
        return data

    def __get_by_page(self, page):
        data = {"page": page}
        req = requests.get(
            url=f"{self.base_url}/",
            headers=self.req_headers,
            json=data,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req.json()

    def get_all(self):
        items = []
        current_page = 1
        more_page = True
        while more_page:
            data = self.__get_by_page(current_page)
            items.extend(data.get("items"))
            total_pages = data.get("meta").get("pagination").get("total_pages") or 0
            if current_page == total_pages:
                more_page = False
            current_page += 1
        df = pd.DataFrame.from_records(items)
        return df

    def get(self, uid):
        try:
            req = requests.get(
                url=f"{self.base_url}/{uid}",
                headers=self.req_headers,
                allow_redirects=False,
            )
            req.raise_for_status()
            return req.json()
        except requests.HTTPError as err:
            err_code = err.response.status_code
            err_msg = err.response.json()
            to_print = f"{err_code}: {err_msg}"
            if err_code == 404:
                to_print = f"{self.msg} id (id={uid}) not found."
            print(to_print)

    def patch(self, data):
        data = self.__values_format(data)
        try:
            uid = data["id"]
            req = requests.put(
                url=f"{self.base_url}/{uid}",
                headers=self.req_headers,
                json=data,
                allow_redirects=False,
            )
            req.raise_for_status()
            # Message success
            to_print = f"{self.msg} (id={uid}) updated."
            if self.model_name == "users":
                email = data["email"]
                to_print = f"User '{email}' (id={uid}) updated."
            print(to_print)
        except requests.HTTPError as err:
            err_code = err.response.status_code
            err_msg = err.response.json()
            to_print = f"{err_code}: {err_msg}"
            if err_code == 404:
                to_print = f"{self.msg} id (id={uid}) not found."
            print(to_print)

    def send(self, data):
        data = self.__values_format(data)
        try:
            req = requests.post(
                url=f"{self.base_url}/",
                headers=self.req_headers,
                json=data,
                allow_redirects=False,
            )
            req.raise_for_status()
            try:
                res = req.json()
                uid = res["id"]
                print(f"{self.msg} successfully created (id={uid})!")
                return uid
            except ValueError:
                print("Send successfull ! No json returned")
        except requests.HTTPError as err:
            err_code = err.response.status_code
            err_msg = err.response.json()
            to_print = f"{err_code}: {err_msg}"
            print(to_print)

    def delete(self, uid):
        try:
            req = requests.delete(
                url=f"{self.base_url}/{uid}",
                headers=self.req_headers,
                allow_redirects=False,
            )
            req.raise_for_status()
            # Message success
            print(f"{self.msg} id (id={uid}) deleted.")
        except requests.HTTPError as err:
            err_code = err.response.status_code
            err_msg = err.response.json()
            to_print = f"{err_code}: {err_msg}"
            if err_code == 404:
                to_print = f"{self.msg} id (id={uid}) not found."
            print(to_print)


class User(TKCRUD):
    def update(
        self,
        uid,
        email=None,
        password=None,
        first_name=None,
        last_name=None,
        company=None,
    ):
        data = {
            "id": uid,
            "email": email,
            "password": password,
            "first_name": first_name,
            "last_name": last_name,
            "company": company,
        }
        res = self.patch(data)
        return res

    def create(self, email, password, first_name=None, last_name=None, company=None):
        data = {
            "email": email,
            "password": password,
            "first_name": first_name,
            "last_name": last_name,
            "company": company,
        }
        res = self.send(data)
        return res


class Enrollment(TKCRUD):
    def update(self, uid, activated=None, expired=None):
        data = {
            "id": uid,
            "activated_at": activated,
            "expiry_date": expired,
        }
        res = self.patch(data)
        return res

    def create(self, course_id, user_id, activated, expired):
        data = {
            "course_id": course_id,
            "user_id": user_id,
            "activated_at": activated,
            "expiry_date": expired,
        }
        res = self.send(data)
        return res


class Courses(TKCRUD):
    def get_chapters(self, uid):
        req = requests.get(
            url=f"{self.base_url}/{uid}/chapters",
            headers=self.req_headers,
            allow_redirects=False,
        )
        req.raise_for_status()
        return req.json()


class Thinkific(InDriver, OutDriver):
    base_url = os.environ.get(
        "THINKIFIC_API_URL", "https://api.thinkific.com/api/public/v1"
    )
    api_token = None
    subdomain = None

    def connect(self, api_token, subdomain):
        # Init thinkific attribute
        self.token = api_token
        self.subdomain = subdomain

        # Init end point
        self.users = User(f"{self.base_url}/users", self.subdomain, self.token)
        self.enrollments = Enrollment(
            f"{self.base_url}/enrollments", self.subdomain, self.token
        )
        self.courses = Courses(f"{self.base_url}/courses", self.subdomain, self.token)
        self.groups = TKCRUD(f"{self.base_url}/groups", self.subdomain, self.token)
        self.group_users = TKCRUD(
            f"{self.base_url}/group_users", self.subdomain, self.token
        )

        # Set connexion to active
        self.connected = True
        return self



from naas_drivers.driver import OutDriver
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders
from typing import Any, Dict, cast
import pandas as pd
import smtplib
from imap_tools import MailBox, A, AND, MailMessage
from datetime import datetime, date
from typing import Literal, Union


class Email(OutDriver):
    """
    Connector for sending email from an authenticated email service over SMTP.

    Args:
        - username (str): username for email service
        - password (str): password for email service
        - email_from (str, optional): the email address to send from
        - smtp_server (str, optional): the hostname of the SMTP server;
            defaults to http://smtp.sendgrid.net/
        - smtp_port (int, optional): the port number of the SMTP server; defaults to 465
        - smtp_type (str, optional): either SSL or STARTTLS; defaults to SSL
    """

    def connect(
        self,
        username: str,
        password: str,
        email_from: str = "bob@cashstory.com",
        smtp_server: str = "smtp.sendgrid.net",
        smtp_port: int = 465,
        smtp_type: str = "SSL",
    ):
        self.username = username
        self.password = password
        self.email_from = email_from
        self.smtp_server = smtp_server
        self.smtp_port = smtp_port
        self.smtp_type = smtp_type
        self.connected = True
        return self

    def get_attachments(self, uid):
        with MailBox(self.smtp_server).login(self.username, self.password) as mailbox:
            attachments = []
            for msg in mailbox.fetch(A(uid=uid)):
                for att in msg.attachments:
                    attachments.append(
                        {
                            "filename": att.filename,
                            "payload": att.payload,
                            "content_id": att.content_id,
                            "content_type": att.content_type,
                            "content_disposition": att.content_disposition,
                            "part": att.part,
                            "size": att.size,
                        }
                    )
        return pd.DataFrame.from_records(attachments)

    def get_mailbox(self, box=""):
        with MailBox(self.smtp_server).login(self.username, self.password) as mailbox:
            return mailbox.folder.list(box)

    def set_seen(self, uid, status=True):
        with MailBox(self.smtp_server).login(self.username, self.password) as mailbox:
            mailbox.seen(mailbox.fetch(AND(uid=uid)), status)

    def set_flag(self, uid, name, status=True):
        with MailBox(self.smtp_server).login(self.username, self.password) as mailbox:
            flags = name.upper()
            mailbox.flag(mailbox.fetch(AND(uid=uid)), flags, status)

    def status(self, box="INBOX"):
        with MailBox(self.smtp_server).login(self.username, self.password) as mailbox:
            return mailbox.folder.status(box)

    def get(self, box="INBOX", criteria="ALL", limit=None, mark=None):
        emails = []
        mark_seen = False
        mail_filter = "ALL"
        if mark and mark == "seen":
            mark_seen = True
        if criteria and criteria == "seen":
            mail_filter = AND(seen=True)
        elif criteria and criteria == "unseen":
            mail_filter = AND(seen=False)
        with MailBox(self.smtp_server).login(
            self.username, self.password, initial_folder=box
        ) as mailbox:
            emails = []
            for msg in mailbox.fetch(mail_filter, limit=limit, mark_seen=mark_seen):
                parsed = {
                    "uid": msg.uid,
                    "subject": msg.subject,
                    "from": msg.from_values,
                    "to": list(msg.to_values),
                    "cc": list(msg.cc_values),
                    "bcc": list(msg.bcc_values),
                    "reply_to": list(msg.reply_to_values),
                    "date": msg.date,
                    "text": msg.text,
                    "html": msg.html,
                    "flags": msg.flags,
                    "headers": msg.headers,
                    "size_rfc822": msg.size_rfc822,
                    "size": msg.size,
                    "obj": msg.obj,
                    "attachments": len(msg.attachments),
                }
                emails.append(parsed)
                if mark and mark == "archive":
                    mailbox.delete([msg.uid])
            return pd.DataFrame.from_records(emails)

    def get_emails_by_subject(self, subject: str, exact: bool = False):
        """
        Get emails with a given subject. Matches all emails which subjects contain the argument. Case-insensitive.
        Args:
            subject (str): the subject to match
            exact (bool, optional): whether to match the exact subject; defaults to False
        Returns:
            pd.DataFrame
        """
        with MailBox(self.smtp_server).login(self.username, self.password) as mailbox:
            emails = []
            # subject = subject returns case-insensitive substring matches
            for msg in mailbox.fetch(AND(subject=subject)):
                if exact:
                    if msg.subject == subject:
                        parsed = self.__parse_message(msg)
                        emails.append(parsed)
                    else:
                        continue
                else:
                    parsed = self.__parse_message(msg)
                    emails.append(parsed)
            return pd.DataFrame.from_records(emails)

    def get_emails_by_date(
        self,
        date: Union[datetime, date],
        condition=Literal["on", "before", "after", "before or on", "after or on"],
    ):
        """
        Get emails with a given date.
        Args:
            date (datetime): the date to match
            condition (str, optional): the condition to match the date; defaults to "on".
            Possible values are: "on", "before", "after or on".
        Returns:
            pd.DataFrame
        """
        with MailBox(self.smtp_server).login(self.username, self.password) as mailbox:
            emails = []
            query = ""
            if condition == "on":
                query = AND(date=date)
            elif condition == "before":
                query = AND(date_lt=date)
            elif condition == "after or on":
                query = AND(date_gte=date)

            for msg in mailbox.fetch(query):
                parsed = self.__parse_message(msg)
                emails.append(parsed)
            return pd.DataFrame.from_records(emails)

    def get_emails_by_sender(self, sender: str, exact: bool = False):
        """
        Get emails with a given sender. Matches all emails which senders contain the argument. Case-insensitive.
        Args:
            sender (str): the sender to match (e.g Bob bob@cashstory.com)
            exact (bool, optional): whether to match the exact sender; defaults to False
        Returns:
            pd.DataFrame
        """
        with MailBox(self.smtp_server).login(self.username, self.password) as mailbox:
            emails = []
            # sender = sender returns case-insensitive substring matches
            for msg in mailbox.fetch(AND(from_=sender)):
                if exact:
                    if msg.from_ == sender:
                        parsed = self.__parse_message(msg)
                        emails.append(parsed)
                    else:
                        continue
                else:
                    parsed = self.__parse_message(msg)
                    emails.append(parsed)
            return pd.DataFrame.from_records(emails)

    def get_emails_by_imap_query(self, query: str):
        """
        Get emails with a given IMAP query. You shoud import AND, OR, NOT from imap_query.
        E.g AND(subject="hello", from_=bob@cashstory.com") would return all emails with the
        subject "hello" and the sender bob@cashstory.com.
        Args:
            query (str): the IMAP query to match
        Returns:
            pd.DataFrame
        """
        with MailBox(self.smtp_server).login(self.username, self.password) as mailbox:
            emails = []
            for msg in mailbox.fetch(query):
                parsed = self.__parse_message(msg)
                emails.append(parsed)
            return pd.DataFrame.from_records(emails)

    def __parse_message(self, message: MailMessage):
        parsed = {
            "uid": message.uid,
            "subject": message.subject,
            "from": message.from_values,
            "to": list(message.to_values),
            "cc": list(message.cc_values),
            "bcc": list(message.bcc_values),
            "reply_to": list(message.reply_to_values),
            "date": message.date,
            "text": message.text,
            "html": message.html,
            "flags": message.flags,
            "headers": message.headers,
            "size_rfc822": message.size_rfc822,
            "size": message.size,
            "obj": message.obj,
            "attachments": len(message.attachments),
        }
        return parsed

    def send(
        self,
        email_to: str,
        subject: str,
        content: str = "",
        files: Dict[str, Any] = None,
    ) -> None:
        """
        Method which sends an email.

        Args:
            - email_to (str): the destination email address to send the message to
            - subject (str): the subject of the email
            - content (str, Optional): the contents of the email
            - files (Dict, optional): Dict of attachments to send

        Returns:
            - None
        """
        self.check_connect()
        email_to = cast(str, email_to)

        contents = MIMEMultipart()
        contents.attach(MIMEText(cast(str, content), "plain"))

        contents["Subject"] = Header(subject, "UTF-8")
        contents["From"] = self.email_from
        contents["To"] = email_to
        if files:
            for k, v in files.items():
                p = MIMEBase("application", "octet-stream")
                with open(v, "rb") as f:
                    p.set_payload(f.read())
                encoders.encode_base64(p)
                p.add_header("Content-Disposition", f"attachment; filename= {k}")
                contents.attach(p)

        message = contents.as_string()

        if self.smtp_type == "SSL":
            server = smtplib.SMTP_SSL(self.smtp_server, self.smtp_port)
        elif self.smtp_type == "STARTTLS":
            server = smtplib.SMTP(self.smtp_server, self.smtp_port)
            server.starttls()
        else:
            raise ValueError("Please set smtp_type to SSL or STARTTLS")

        server.login(self.username, self.password)
        try:
            server.sendmail(self.email_from, email_to, message)
        finally:
            server.quit()
            print("email send")



import pandas as pd
import requests
import time
import urllib
import secrets
import naas
from naas_drivers.tools.emailbuilder import EmailBuilder
from naas_drivers.tools.naas_auth import NaasAuth

emailbuilder = EmailBuilder()
naasauth = NaasAuth()

LINKEDIN_API = "https://3hz1hdpnlf.execute-api.eu-west-1.amazonaws.com/prod"
DATE_FORMAT = "%Y-%m-%d"
DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
TIME_SLEEP = secrets.randbelow(5) + 5
HEADERS = {"Content-Type": "application/json"}
EMAIL_COOKIES = "⚠️ Naas.ai - Update your Linkedin cookies"


class LinkedIn:
    deprecated = True

    @staticmethod
    def get_user_email():
        email = None
        user = naasauth.connect().user.me()
        email = user.get("username")
        return email

    @staticmethod
    def email_linkedin_limit(email):
        content = {
            "header_naas": (
                "<a href='https://www.naas.ai/'>"
                "<img align='center' width='30%' target='_blank' style='border-radius:5px;'"
                "src='https://landen.imgix.net/jtci2pxwjczr/assets/5ice39g4.png?w=160'"
                "alt='Please provide more information.'/>"
                "</a>"
            ),
            "txt_0": emailbuilder.text(
                "Hi there,<br><br>"
                "Your LinkedIn cookies needs to be renewed.<br><br>"
                "Please go to naas and update them in your notebook 'Setup LinkedIn'.<br>"
            ),
            "button": emailbuilder.button(
                f"https://app.naas.ai/user/{email}/", "Go to Naas"
            ),
            "signature": "Naas Team",
            "footer": emailbuilder.footer_company(naas=True),
        }
        email_content = emailbuilder.generate(display="iframe", **content)
        return email_content

    @staticmethod
    def send_email_renewed_cookies():
        email = LinkedIn.get_user_email()
        email_content = LinkedIn.email_linkedin_limit(email)
        naas.notification.send(
            email_to=email, subject=EMAIL_COOKIES, html=email_content
        )

    @staticmethod
    def manage_api_error(res):
        if res.status_code != 200:
            if int(res.status_code) == 302:
                LinkedIn.send_email_renewed_cookies()
                raise requests.TooManyRedirects(res.status_code, res.text)
            else:
                raise BaseException(res.status_code, res.text)

    def connect(
        self,
        li_at: str = None,
        jessionid: str = None,
        li_a: str = None,
    ):
        # Init lk attribute
        self.li_at = li_at
        self.jessionid = jessionid
        self.li_a = li_a

        # Init cookies
        self.cookies = {
            "li_at": self.li_at,
            "JSESSIONID": f'"{self.jessionid}"',
            "li_a": self.li_a,
        }

        # Init headers
        self.headers = {"Content-Type": "application/json"}

        # Init end point
        self.leads = Leads(self.cookies, self.headers)

        # Set connexion to active
        self.connected = True
        return self


class Leads(LinkedIn):
    def __init__(self, cookies, headers):
        LinkedIn.__init__(self)
        self.cookies = cookies
        self.headers = headers

    def get_list(self, url, start=0, count=100, limit=1000):
        # Init
        df = pd.DataFrame()
        if limit != -1 and limit < count:
            count = limit
        # Requests API
        while True:
            params = {
                "url": url,
                "start": start,
                "count": count,
            }
            req_url = f"{LINKEDIN_API}/leads/getList?{urllib.parse.urlencode(params, safe='(),')}"
            res = requests.post(req_url, json=self.cookies, headers=self.headers)
            res.raise_for_status()

            # Manage LinkedIn API errors
            LinkedIn.manage_api_error(res)

            # Get json result
            res_json = res.json()
            if len(res_json) == 0:
                break
            tmp_df = pd.DataFrame(res_json)
            df = pd.concat([df, tmp_df], axis=0)
            start += count
            if limit != -1 and start >= limit:
                break
            elif limit != -1 and limit - start < count:
                count = limit - start
            time.sleep(TIME_SLEEP)
        return df.reset_index(drop=True)
